/*
    Copyright 2013 Jorge Paniagua

    This file is part of Recro GameEngine.

    Recro GameEngine is free software: you can redistribute it and/or modify
    it under the terms of the Lesser GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    Recro GameEngine is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Lesser GNU General Public License for more details.

    You should have received a copy of the Lesser GNU General Public License
    along with Recro GameEngine. If not, see <http://www.gnu.org/licenses/>.

    Any questions about the software write to gxsoft6@gmail.com

    The tutorials are in http://gxsoftware.jimdo.com/tutorials/
*/

#pragma once

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_ttf.h>
#include "RecroMouse.h"
#include "RecroImage.h"
#include "RecroMusic.h"
#include "RecroText.h"
#include "RecroSprite.h"
#include "RecroSound.h"
#include "RecroKey.h"
#include "RecroEvent.h"

class RecroGame
{
    private:
        int FrameRate;  //aqui se almacenara la tasa de frames per second
        int FullScreen; //vale 0 si esta en modo ventana y 1 en fullscreen
        SDL_Surface *BackGround;    //es el fondo de pantalla(no es obligatorio su uso)
        SDL_Surface *Buffer;
        SDL_Surface *Screen;    //esta superficie armacena todo el juego
        SDL_Rect BackGroundCoor;
        int SeeCursor;

    public:
        RecroKey KeyBoard;   //maneja el teclado
        RecroEvent Event;    //armacena eventos
        RecroMouse Mouse;
        RecroMusic Music;
        int Width;
        int Height;
        int Bpp;    //profundidad del color 8,16,24 o 32 bits
        int Stereo; //0 es mono y 1 es estereo
        int Frequency;
        int Chunksize;
        int Channels;   //numero de sonidos que se reproduciran al mismo tiempo(10 por defecto)

        RecroGame();    //constructor de la clase
        void Init();
        void Init(int w, int h);    //lo mismo que init pero definiendo la resolucion
        void Stop();
        void Delay(int mls);
        void GetFrameRate();    //tomar la cantidad de tiempo que ha pasado en el juego
        void LimitFrameRate(int ms);    //limitar el juego el numero de milisegundos
        int GetFPS();   //retorna los fps
        void Refresh(); //mostrar todo el juego en pantalla
        void ShowBackGround();
        void LoadBackGround(const char *path);
        void ChangeWindowMode();    //cambia entre fullscreen y modo ventana
        void ChangeWindowRes(int w, int h); //cambiar resolucion
        void ChangeBpp(int bpp);
        void ChangeCursorMode();    //mostrar o ocurtar cursor
        void ChangeWindowTitle(const char *title);
        void ChangeWindowIcon(const char *icon);
        void SlowMotionMusic();
        void Show(RecroImage image);
        void Show(RecroText text);
        void Show(RecroText text, int rendertype);
        void Show(RecroSprite sprite,int frame=1);
        bool SpriteColicion(RecroSprite a, RecroSprite b);
        bool SpriteColicion(RecroSprite a, RecroImage b);
        bool SpriteColicion(RecroSprite a, RecroText b);
        bool ImageColicion(RecroImage a, RecroImage b);
        bool ImageColicion(RecroImage a, RecroSprite b);
        bool ImageColicion(RecroImage a, RecroText b);
        bool TextColicion(RecroText a, RecroText b);
        bool TextColicion(RecroText a, RecroSprite b);
        bool TextColicion(RecroText a, RecroImage b);
        bool CursorColicion(RecroSprite b);
        bool CursorColicion(RecroImage b);
        bool CursorColicion(RecroText b);
        ~RecroGame(){Stop();}
};
